
/*  -*- MaTX -*-
 *
 *  NAME
 *      th_lti2() - Convert theta-parameter to transfer function
 *
 *  SYNOPSIS
 *       G = th_lti2(th, m, n, p, scale, ...)
 *          RaMatrix G;
 *          Matrix th;
 *          Integer m;  number of inputs
 *          Integer n;  number of states
 *          Integer p;  number of outputs
 *          Real scale; scaling parameter
 *
 *  DESCRIPTION
 *      th_lti2() converts theta parameter, th,  to transfer function 
 *      matrix, G;
 *
 *      m    : number of inputs
 *      n    : number of states
 *      p    : number of outputs
 *      scale: scaling parameter
 *
 *      It is assumed that all the elements of the transfer function 
 *      have the common denominator such as
 *
 *      y(k) = G(q)u(k) + w(k)
 *                or
 *      y(k) = G(z)u(k) + w(k)
 *

G(q)=[[ b1_11 q^-1 + ... + b1_1n q^-n         bm_11 q^-1 + ... + bm_1n q^-n ]
      [-------------------------------, ..., -------------------------------]
      [ 1 + a_1 q^-1 + ... + a_n q^-n         1 + a_1 q^-1 + ... + a_n q^-n ]
      [                                                                     ]
      [                                                                     ]
      [ b1_p1 q^-1 + ... + b1_pn q^-n         bm_p1 q^-1 + ... + bm_pn q^-n ]
      [-------------------------------, ..., -------------------------------]
      [ 1 + a_1 q^-1 + ... + a_n q^-n         1 + a_1 q^-1 + ... + a_n q^-n ]]

G(z)=[[   b1_11 z^n-1 + ... + b1_1n             bm_11 z^n-1 + ... + bm_1n   ]
      [-------------------------------, ..., -------------------------------]
      [  z^n + a_1 z^n-1 + ... + a_n           z^n + a_1 z^n-1 + ... + a_n  ]
      [                                                                     ]
      [                                                                     ]
      [   b1_p1 z^n-1 + ... + b1_pn             bm_p1 z^n-1 + ... + bm_pn   ]
      [-------------------------------, ..., -------------------------------]
      [  z^n + a_1 q^z-n1 + ... + a_n          z^n + a_1 z^n-1 + ... + a_n  ]]

 *
 * or in polynomial form:
 *
 *      [[y1(k)]       [[u1(k)]
 * A(q)  [y2(k)] = B(q) [u2(k)]  + w(k)
 *       [.....]        [.....]
 *       [yp(k)]]       [um(k)]] 
 * 
 * A(q) = diag(1 + a_1 q^-1 + ... a_n q^-n,
 *                       1 + a_1 q^-1 + ... a_n q^-n,
 *                                 ...........................,
 *                                           1 + a_1 q^-1 + ... a_n q^-n)
 * 
 * B(q) = [[1 + b1_11 q^-1 + ... b1_1n q^-n, 1 + b1_21 q^-1 + ... b1_2n q^-n
 *          ..............................., 1 + b1_m1 q^-1 + ... b1_mn q^-n]
 *         [1 + b2_11 q^-1 + ... b2_1n q^-n, 1 + b2_21 q^-1 + ... b2_2n q^-n
 *          ..............................., 1 + b2_m1 q^-1 + ... b2_mn q^-n]
 *         [................................................................]
 *         [1 + bp_11 q^-1 + ... bp_1n q^-n, 1 + bp_21 q^-1 + ... bp_2n q^-n
 *          ..............................., 1 + bp_m1 q^-1 + ... bp_mn q^-n]]
 *
 * or in theta-parameter form:
 * 
 *     [[y1(k)]
 *      [y2(k)] = phi(k)#*th + w(k)
 *      [.....]
 *      [yp(k)]]
 *
 *	  th  : (n + p*n*m) x 1
 *	  phi : p x (n + p*n*m)
 *
 *  th = [[a_1]  th1 = [[b1_11] th2 = [[b1_21]   ... thp = [[b1_p2]
 *        [a_2]         [b1_12]        [b1_22]   ...        [b1_p2]
 *        [...]         [ ....]        [ ....]   ...        [ ... ]
 *        [a_n]         [b1_1n]        [b1_2n]   ...        [b1_pn]
 *        [th1]         [ ....]        [ ....]   ...        [ ... ]
 *        [th2]         [bm_11]        [bm_21]   ...        [bm_p2]
 *        [...]         [bm_12]        [bm_22]   ...        [bm_p2]
 *        [thp]]        [ ....]        [ ....]   ...        [ ... ]
 *                      [bm_1n]]       [bm_2n]]  ...        [bm_pn]]
 *
 *  phi(k) = [[YY(k)]
 *            [UU(k)]]
 *
 *  UU(k) = diag([[ u_1(k-1)]   [ u_1(k-1)]   ...,  [ u_1(k-1)]
 *                [  ....   ]   [  ....   ]   ...,  [  ....   ]
 *                [ u_1(k-n)]   [ u_1(k-n)]   ...,  [ u_1(k-n)]
 *                [  ....   ]   [   ....  ]   ...,  [  ....   ]
 *                [  ....   ]   [   ....  ]   ...,  [  ....   ]
 *                [ u_m(k-1)]   [ u_m(k-1)]   ...,  [ u_m(k-1)]
 *                [  ....   ]   [  ....   ]   ...,  [  ....   ]
 *                [ u_m(k-n)]], [ u_m(k-n)]], ...,  [ u_m(k-n)]])
 *
 *  YY(k) = [[-y_1(k-1), -y_2(k-1), ..., -y_p(k-1)]
 *           [  ....   ,   ....   , ...,   ....   ]
 *           [-y_1(k-n), -y_2(k-n), ..., -y_p(k-n)]]
 *
 */

Func RaMatrix th_lti2(th, m, n, p, scale, ...)
	Matrix th;
	Integer m;
	Integer n;
	Integer p;
	Real scale;
{
	Integer i, j, ii, jj;
	Polynomial N, D;
	RaMatrix G;

	error(nargchk(4, 5, nargs, "th_lti2"));
	
	if (nargs == 4) {
		scale = 1.0;
	}

	G = RaMatrix(Z(p, m));
	if (version() == 4) {
		D = Polynomial(fliplr([[1][th(1:n,1)]]#));
	} else {
		D = Polynomial([[1][th(1:n,1)]]#);
	}
	
	for (i = 1; i <= p; i++) {
		ii = n + (i-1)*n*m;
		for (j = 1; j <= m; j++) {
			jj = ii + n*(j-1);
			if (version() == 4) {
				N = Polynomial(fliplr(th(jj+1:jj+n,1)#)) * scale;
			} else {
				N = Polynomial(th(jj+1:jj+n,1)#) * scale;
			}
			G(i,j) = N/D;
		}
	}
	return G;
}

