
/*  -*- MaTX -*-
 *
 *  NAME
 *      quadf() - Numerical evaluation of integrals (quadrature)
 *
 *  SYNOPSIS
 *      {q,cnt} = quadf(funfcn,a,b)
 *          Real q;
 *          Integer cnt;
 *          Matrix funfnc();
 *          Real a,b;
 *
 *      {q,cnt} = quadf(funfcn,a,b,tol)
 *          Real q;
 *          Integer cnt;
 *          Matrix funfnc();
 *          Real a,b;
 *          Real tol;
 *
 *      {q,cnt} = quadf(funfcn,a,b,tol,trc)
 *          Real q;
 *          Integer cnt;
 *          Matrix funfnc();
 *          Real a,b;
 *          Real tol;
 *          Integer trc;
 *
 *  DESCRIPTION
 *      quadf() approximates the integral of funfcn(x) from a to b
 *      to within a relative error of 1.0e-3.  Function funfcn() must
 *      return {q,cnt} a vector of output values if given a vector of
 *      input values.
 *
 *      quadf(...,tol) integrates to a relative error of tol.
 *      q = Inf is returned if an excessive recursion level is reached,
 *      indicating a possibly singular integral.
 *
 *      quadf(...,trc) integrates to a relative error of tol and
 *      traces the function evaluations with a point plot of the integrand.
 *
 *      quadf() uses an adaptive recursive Simpson's rule.
 *      quadf() also returns a function evaluation count.
 *
 *  SEE ALSO
 *     quad8() and quad().
 *
 */

Func List quadf(funfcn,a,b,tol,trc, ...)
	Matrix funfcn();
	Real a;
	Real b;
	Real tol;
	Integer trc;
{
	Integer cnt,lev,win;
	Real q,fa,fb,fc,c;
	Matrix lims,x,y,X,F;
	Array inda;
	Index ind;

	List quadstp();

	error(nargchk(3, 5, nargs, "quadf"));

	win = mgplot_cur_win();

	if (nargs == 3) {
		tol = 1.0e-3;
		trc = 0;
	} else if (nargs == 4) {
		trc = 0;
	}

	c = (a + b)/2;

	// Top level initialization
	x = [[a], [b], [c], [a:(b-a)/10:b]];
	y = funfcn(x);
	fa = y(1);
	fb = y(2);
	fc = y(3);
	if (trc) {
		lims = [min(x) max(x) min(y) max(y)];
		ind = find( ! isfinite(lims));
		if (length(ind) != 0) {
			inda = ind;
			lims(ind) = 1.0e30 * (-ONE(inda) .^ rem(inda,2));
		}
		mgreplot(win, [c b], [fc fb], {""}, {"w dots 3"});
	}
	lev = 1;

	// Adaptive, recursive Simpson's quadrature
	{q, cnt, X, F} = quadstp(funfcn,a,b,tol,lev,fa,fc,fb,2.0^1023,trc);
//	{q, cnt, X, F} = quadstp(funfcn,a,b,tol,lev,fa,fc,fb,Inf,trc);

#ifdef VC40
	mgreplot(win, X, F, {""}, {"w points 3"});
#endif

	cnt = cnt + 3;
	return {q, cnt};
}


/*  -*- MaTX -*-
 *
 *  NAME
 *      quadstp() - Recursive function used by quad()
 *
 *  SYNOPSIS
 *     {q, cnt, x, f} = quadstp(funfcn,a,b,tol,lev,fa,fc,fb,q0,trc)
 *         Real q;
 *         Integer cnt;
 *         Matrix funfcn();
 *         Real a,b,tol;
 *         Integer lev;
 *         Real fa,fc,fb,q0;
 *         Integer trc;
 *
 *  DESCRIPTION
 *      quadstp() tries to approximate the integral of funfcn(x)
 *      from a to b to within a relative error of tol.
 *      The arguments are generated by quad or by the recursion.
 *
 *      lev : Recursion level
 *       fa : f(a)
 *       fc : f((a+b)/2)
 *       fb : f(b)
 *       q0 : Approximate value of the integral
 * 
 *  SEE ALSO
 *      quad() and quad8().
 *
 */

Func List quadstp(funfcn,a,b,tol,lev,fa,fc,fb,q0,trc)
	Matrix funfcn();
	Real a;
	Real b;
	Real tol;
	Integer lev;
	Real fa;
	Real fc;
	Real fb;
	Real q0;
	Integer trc;
{
	Integer cnt,cnt1,cnt2,lev_max;
#ifndef VC40
	Integer win;
#endif
	Real q,q1,q2,c,h;
	Matrix f,x,X,F;
	
	lev_max = 10;
#ifndef VC40
	win = mgplot_cur_win();
#endif

	if (lev > lev_max) {
		print "Recursion level limit reached in quad.  Singularity likely.\n";
		q = q0;
		cnt = 0;
		c = (a + b)/2;
		if (trc) {
			x = [c];
			f = funfcn(x);
#ifndef VC40
			mgreplot(win, x, f, {""}, {"w dots 3"});
#endif
		}
	} else {
		// Evaluate function at midpoints of left and right half intervals
		h = b - a;
		c = (a + b)/2;
		x = [a+h/4, b-h/4];
		f = funfcn(x);
		if (trc) {
#ifndef VC40
			mgreplot(win, x, f, {""}, {"w points 3"});
#endif
		}
		cnt = 2;
		
		// Simpson's rule for half intervals
		q1 = h*(fa + 4*f(1) + fc)/12;
		q2 = h*(fc + 4*f(2) + fb)/12;
		q = q1 + q2;

		// Recursively refine approximations
		if (abs(q - q0) > tol*abs(q)) {
			{q1,cnt1,X,F} = quadstp(funfcn,a,c,tol/2,lev+1,fa,f(1),fc,q1,trc);
			x = [x X];
			f = [f F];
			{q2,cnt2,X,F} = quadstp(funfcn,c,b,tol/2,lev+1,fc,f(2),fb,q2,trc);
			x = [x X];
			f = [f F];
			q = q1 + q2;
			cnt = cnt + cnt1 + cnt2;
		}
		
	}
	return {q, cnt, x, f};
}


